@@ -59,6 +59,9 @@ module Data.Text.Internal.Builder
 
 import Control.Monad.ST (ST, runST)
 import Data.Monoid (Monoid(..))
+#if MIN_VERSION_base(4,9,0)
+import Data.Semigroup (Semigroup(..))
+#endif
 import Data.Text.Internal (Text(..))
 import Data.Text.Internal.Lazy (smallChunkSize)
 import Data.Text.Unsafe (inlineInterleaveST)
@@ -89,12 +92,22 @@ newtype Builder = Builder {
                 -> ST s [S.Text]
    }
 
+#if MIN_VERSION_base(4,9,0)
+instance Semigroup Builder where
+   (<>) = append
+   {-# INLINE (<>) #-}
+#endif
+
 instance Monoid Builder where
    mempty  = empty
    {-# INLINE mempty #-}
+#if MIN_VERSION_base(4,9,0)
+   mappend = (<>) -- future-proof definition
+#else
    mappend = append
+#endif
    {-# INLINE mappend #-}
-   mconcat = foldr mappend mempty
+   mconcat = foldr mappend Data.Monoid.mempty
    {-# INLINE mconcat #-}
 
 instance String.IsString Builder where
