@@ -1,5 +1,7 @@
 {-# LANGUAGE OverloadedStrings, CPP #-}
-#if __GLASGOW_HASKELL__ >= 702
+#if __GLASGOW_HASKELL__ >= 704
+{-# LANGUAGE Safe #-}
+#elif __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE Trustworthy #-}
 #endif
 
@@ -54,7 +56,7 @@ decimal :: Integral a => Reader a
 {-# SPECIALIZE decimal :: Reader Int32 #-}
 {-# SPECIALIZE decimal :: Reader Int64 #-}
 {-# SPECIALIZE decimal :: Reader Integer #-}
-{-# SPECIALIZE decimal :: Reader Word #-}
+{-# SPECIALIZE decimal :: Reader Data.Word.Word #-}
 {-# SPECIALIZE decimal :: Reader Word8 #-}
 {-# SPECIALIZE decimal :: Reader Word16 #-}
 {-# SPECIALIZE decimal :: Reader Word32 #-}
