@@ -7,48 +7,46 @@
 module Tests.QuickCheckUtils
     (
       genUnicode
-    , genUnicodeWith
-    , ascii
-    , plane0
-    , plane1
-    , plane2
-    , plane14
-    , planes
-
     , unsquare
     , smallArbitrary
 
-    , NotEmpty (..)
+    , BigBounded(..)
+    , BigInt(..)
+    , NotEmpty(..)
 
-    , Small (..)
+    , Small(..)
     , small
 
+    , Precision(..)
+    , precision
+
     , integralRandomR
 
-    , DecodeErr (..)
+    , DecodeErr(..)
     , genDecodeErr
 
-    , Stringy (..)
+    , Stringy(..)
     , eq
     , eqP
 
-    , Encoding (..)
+    , Encoding(..)
 
     , write_read
     ) where
 
+import Control.Applicative ((<$>))
 import Control.Arrow (first, (***))
 import Control.DeepSeq (NFData (..), deepseq)
 import Control.Exception (bracket)
-import Data.Bits ((.&.))
-import Data.Char (chr)
 import Data.String (IsString, fromString)
 import Data.Text.Foreign (I16)
+import Data.Text.Lazy.Builder.RealFloat (FPFormat(..))
 import Data.Word (Word8, Word16)
 import Debug.Trace (trace)
-import System.Random (Random (..), RandomGen)
-import Test.QuickCheck hiding (Small (..), (.&.))
+import System.Random (Random(..), RandomGen)
+import Test.QuickCheck hiding (Fixed(..), Small (..), (.&.))
 import Test.QuickCheck.Monadic (assert, monadicIO, run)
+import Test.QuickCheck.Unicode (string)
 import Tests.Utils
 import qualified Data.ByteString as B
 import qualified Data.Text as T
@@ -60,6 +58,14 @@ import qualified Data.Text.Internal.Lazy.Fusion as TLF
 import qualified Data.Text.Lazy as TL
 import qualified System.IO as IO
 
+#if !MIN_VERSION_base(4,4,0)
+import Data.Int (Int64)
+import Data.Word (Word, Word64)
+#endif
+
+genUnicode :: IsString a => Gen a
+genUnicode = fromString <$> string
+
 instance Random I16 where
     randomR = integralRandomR
     random  = randomR (minBound,maxBound)
@@ -73,76 +79,23 @@ instance Arbitrary B.ByteString where
     shrink        = map B.pack . shrink . B.unpack
 
 #if !MIN_VERSION_base(4,4,0)
+instance Random Int64 where
+    randomR = integralRandomR
+    random  = randomR (minBound,maxBound)
+
+instance Random Word where
+    randomR = integralRandomR
+    random  = randomR (minBound,maxBound)
+
 instance Random Word8 where
     randomR = integralRandomR
     random  = randomR (minBound,maxBound)
+
+instance Random Word64 where
+    randomR = integralRandomR
+    random  = randomR (minBound,maxBound)
 #endif
 
-genUnicode :: IsString a => Gen a
-genUnicode = genUnicodeWith planes
-
-genUnicodeWith :: IsString a => [Gen Int] -> Gen a
-genUnicodeWith gens = fmap fromString string
-  where
-    string = sized $ \n ->
-        do k <- choose (0,n)
-           sequence [ char | _ <- [1..k] ]
-
-    excluding :: [a -> Bool] -> Gen a -> Gen a
-    excluding bad gen = loop
-      where
-        loop = do
-          x <- gen
-          if or (map ($ x) bad)
-            then loop
-            else return x
-
-    reserved = [lowSurrogate, highSurrogate, noncharacter]
-    lowSurrogate c = c >= 0xDC00 && c <= 0xDFFF
-    highSurrogate c = c >= 0xD800 && c <= 0xDBFF
-    noncharacter c = masked == 0xFFFE || masked == 0xFFFF
-      where
-        masked = c .&. 0xFFFF
-
-    char = chr `fmap` excluding reserved (oneof gens)
-
-ascii :: Gen Int
-ascii = choose (0,0x7F)
-
-plane0 :: Gen Int
-plane0 = choose (0xF0, 0xFFFF)
-
-plane1 :: Gen Int
-plane1 = oneof [ choose (0x10000, 0x10FFF)
-               , choose (0x11000, 0x11FFF)
-               , choose (0x12000, 0x12FFF)
-               , choose (0x13000, 0x13FFF)
-               , choose (0x1D000, 0x1DFFF)
-               , choose (0x1F000, 0x1FFFF)
-               ]
-
-plane2 :: Gen Int
-plane2 = oneof [ choose (0x20000, 0x20FFF)
-               , choose (0x21000, 0x21FFF)
-               , choose (0x22000, 0x22FFF)
-               , choose (0x23000, 0x23FFF)
-               , choose (0x24000, 0x24FFF)
-               , choose (0x25000, 0x25FFF)
-               , choose (0x26000, 0x26FFF)
-               , choose (0x27000, 0x27FFF)
-               , choose (0x28000, 0x28FFF)
-               , choose (0x29000, 0x29FFF)
-               , choose (0x2A000, 0x2AFFF)
-               , choose (0x2B000, 0x2BFFF)
-               , choose (0x2F000, 0x2FFFF)
-               ]
-
-plane14 :: Gen Int
-plane14 = choose (0xE0000, 0xE0FFF)
-
-planes :: [Gen Int]
-planes = [ascii, plane0, plane1, plane2, plane14]
-
 -- For tests that have O(n^2) running times or input sizes, resize
 -- their inputs to the square root of the originals.
 unsquare :: (Arbitrary a, Show a, Testable b) => (a -> b) -> Property
@@ -160,6 +113,20 @@ instance Arbitrary TL.Text where
     arbitrary = (TL.fromChunks . map notEmpty) `fmap` smallArbitrary
     shrink = map TL.pack . shrink . TL.unpack
 
+newtype BigInt = Big Integer
+               deriving (Eq, Show)
+
+instance Arbitrary BigInt where
+    arbitrary = choose (1::Int,200) >>= \e -> Big <$> choose (10^(e-1),10^e)
+    shrink (Big a) = [Big (a `div` 2^(l-e)) | e <- shrink l]
+      where l = truncate (log (fromIntegral a) / log 2 :: Double) :: Integer
+
+newtype BigBounded a = BigBounded a
+                     deriving (Eq, Show)
+
+instance (Bounded a, Random a, Arbitrary a) => Arbitrary (BigBounded a) where
+    arbitrary = BigBounded <$> choose (minBound, maxBound)
+
 newtype NotEmpty a = NotEmpty { notEmpty :: a }
     deriving (Eq, Ord)
 
@@ -300,6 +267,30 @@ eqP f g s w  = eql "orig" (f s) (g t) &&
             | a =^= b   = True
             | otherwise = trace (d ++ ": " ++ show a ++ " /= " ++ show b) False
 
+instance Arbitrary FPFormat where
+    arbitrary = elements [Exponent, Fixed, Generic]
+
+newtype Precision a = Precision (Maybe Int)
+                    deriving (Eq, Show)
+
+precision :: a -> Precision a -> Maybe Int
+precision _ (Precision prec) = prec
+
+arbitraryPrecision :: Int -> Gen (Precision a)
+arbitraryPrecision maxDigits = Precision <$> do
+  n <- choose (-1,maxDigits)
+  return $ if n == -1
+           then Nothing
+           else Just n
+
+instance Arbitrary (Precision Float) where
+    arbitrary = arbitraryPrecision 11
+    shrink    = map Precision . shrink . precision undefined
+
+instance Arbitrary (Precision Double) where
+    arbitrary = arbitraryPrecision 22
+    shrink    = map Precision . shrink . precision undefined
+
 -- Work around lack of Show instance for TextEncoding.
 data Encoding = E String IO.TextEncoding
 
